Module: Trilogy::DatabaseError

Included in:
ProtocolError
Defined in:
lib/trilogy/error.rb

Overview

DatabaseError was replaced by ProtocolError, but we'll keep it around as an ancestor of ProtocolError for compatibility reasons (e.g. so rescue DatabaseError still works. We can remove this class in the next major release.