Exception: Trilogy::SynchronizationError

Inherits:
BaseError
  • Object
show all
Defined in:
lib/trilogy/error.rb

Instance Attribute Summary

Attributes included from Error

#error_code

Instance Method Summary collapse

Constructor Details

#initialize(message = "This connection is already in use by another thread or fiber") ⇒ SynchronizationError

Returns a new instance of SynchronizationError.



58
59
60
# File 'lib/trilogy/error.rb', line 58

def initialize(message = "This connection is already in use by another thread or fiber")
  super
end