Exception: Trilogy::SynchronizationError
- Defined in:
- lib/trilogy/error.rb
Instance Attribute Summary
Attributes included from Error
Instance Method Summary collapse
-
#initialize(message = "This connection is already in use by another thread or fiber") ⇒ SynchronizationError
constructor
A new instance of SynchronizationError.
Constructor Details
#initialize(message = "This connection is already in use by another thread or fiber") ⇒ SynchronizationError
Returns a new instance of SynchronizationError.
58 59 60 |
# File 'lib/trilogy/error.rb', line 58 def initialize( = "This connection is already in use by another thread or fiber") super end |